---@type Tinkr, Bastion
local Tinkr, Bastion = ...

-- Create a module class for a bastion module

---@class Bastion.Module
---@field name string
---@field enabled boolean
---@field synced function[]
---@field interval? number
---@field nextTick number
local Module = {}
Module.__index = Module

-- __tostring
---@return string
function Module:__tostring()
    return "Bastion.__Module(" .. self.name .. ")"
end

-- Constructor
---@param name string
---@param interval? number
---@return Bastion.Module
function Module:New(name, interval)
    local module = {}
    setmetatable(module, Module)

    module.name = name
    module.enabled = false
    module.synced = {}
    module.interval = interval
    module.nextTick = 0
    return module
end

-- Enable the module
function Module:Enable()
    self.enabled = true
end

-- Disable the module
function Module:Disable()
    self.enabled = false
end

-- Toggle the module
---@return nil
function Module:Toggle()
    if self.enabled then
        self:Disable()
    else
        self:Enable()
    end
end

-- Add a function to the sync list
---@param func function
function Module:Sync(func)
    table.insert(self.synced, func)
end

-- Remove a function from the sync list
---@param func function
function Module:Unsync(func)
    for i = 1, #self.synced do
        if self.synced[i] == func then
            table.remove(self.synced, i)
            break
        end
    end
end

-- Sync
function Module:Tick()
    if self.enabled then
        if Bastion.Tick >= self.nextTick then
            self.nextTick = Bastion.Tick + (self.interval or 1)
            for i = 1, #self.synced do
                self.synced[i]()
            end
        end
    end
end

return Module
