---@type Tinkr, Bastion
local Tinkr, Bastion = ...

-- Create a new SpellBook class
---@class Bastion.SpellBook
---@field spells table<number, Bastion.Spell>
---@field auras { [spellId]: { [spellId]: { spell: Bastion.Spell, source?: Bastion.Unit, target?: Bastion.Unit, lastApplied: number } } }
local SpellBook = {}
SpellBook.__index = SpellBook

-- Constructor
---@return Bastion.SpellBook
function SpellBook:New()
    local self = setmetatable({}, SpellBook)
    self.spells = {}
    --[[ Bastion.Globals.EventManager:RegisterWoWEvent("UNIT_SPELLCAST_SUCCEEDED", function(...)
        self:SpellCastSucceeded(...)
    end) ]]
    return self
end

--[[ function SpellBook:SpellCastSucceeded(...)
    local unit, castGUID, spellId = ...
    if self.spells[spellId] then
        self.spells[spellId].lastCastAt = GetTime()
    end
end ]]

-- Get a spell from the spellbook
---@param id integer
---@return Bastion.Spell
function SpellBook:GetSpell(id)
    local override = FindSpellOverrideByID(id)
    if self.spells[id] == nil then
        self.spells[id] = Bastion.Spell:New(id)
    end

    if override and override ~= id then
        if self.spells[override] == nil then
            self.spells[override] = Bastion.Spell:New(override)
        end
    end

    return self.spells[id]
end

---@param ... integer
---@return Bastion.Spell, ... Spell
function SpellBook:GetSpells(...)
    local spells = {}
    for _, id in ipairs({ ... }) do
        table.insert(spells, self:GetSpell(id))
    end

    return unpack(spells)
end

---@param aura Bastion.Spell
---@return Bastion.Spell[]
function SpellBook:GetAuraSpells(aura)
    local spells = {}
    for _, spell in pairs(self.spells) do
        if spell:CheckAuraStatus(aura) then
            table.insert(spells, spell)
        end
    end

    return spells
end

---@param ... integer
---@return Bastion.List
function SpellBook:GetList(...)
    local spells = {}
    for _, id in ipairs({ ... }) do
        table.insert(spells, self:GetSpell(id))
    end

    return Bastion.List:New(spells)
end

---@param name string
---@return Bastion.Spell
function SpellBook:GetSpellByName(name)
    local _, rank, icon, castTime, minRange, maxRange, spellID, originalIcon = GetSpellInfo(name)
    return self:GetSpell(spellID)
end

---@param id integer
function SpellBook:GetIfRegistered(id)
    if self.spells[id] then
        return self.spells[id]
    end

    if self.spells[FindSpellOverrideByID(id)] then
        return self.spells[FindSpellOverrideByID(id)]
    end

    return false
end

return SpellBook
